//
//	Create an events_Holder.
//

var/events_Holder/gEventHolder	= new

//
//	Create your events.
//

//////////////////////////////////////////////////////////////////////////////////////////////////
events_Event/Demo/Smile

	//============================================================================================
	events_Perform(events_Holder/Holder,events_Trigger/Trigger,Source,Target,Argument)
		//	This is a contrived example of how to use Argument.
		//	However, the point is that events_Holder.events_AddEvent() passes it automatically.
		world	<< "[Source] smiles at [Target]. [Argument]"
		world	<< "\tNote how this event happens whenever someone is hugged."
		return TRUE

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
events_Event/Demo/Mean

	//============================================================================================
	events_Perform(events_Holder/Holder,events_Trigger/Trigger,Source,Target,Argument)
		//	Whether or not you should use Argument in your events is up to you.
		//	It is completely optional.
		world	<< "[Target] yells: [Source] is mean!"
		world	<< "\tNote how this event is added the first time someone is punched and removed when someone is hugged."
		return TRUE

//////////////////////////////////////////////////////////////////////////////////////////////////

var/events_Event/Demo
	Mean/gEventMean			= new
	Smile/gEventSmile		= new

//
//	Create your triggers.
//

var/events_Trigger
	gTriggerHugs	= new	//	Source hugged someone.
	gTriggerHugged	= new	//	Source has been hugged.
	gTriggerPunches	= new	//	Source punched someone.
	gTriggerPunched	= new	//	Source has been punched.

//
//	Call the events_Triggered() procs for your triggers in your code where you want events activated,
//	added, or removed.
//

//////////////////////////////////////////////////////////////////////////////////////////////////
mob

	//============================================================================================
	verb
		//----------------------------------------------------------------------------------------
		hug(Person as mob in world)
			world	<< "[src] hugs [Person]."

			gTriggerHugs.events_Triggered(gEventHolder,src,Person)
			gTriggerHugged.events_Triggered(gEventHolder,Person,src)

		//----------------------------------------------------------------------------------------
		punch(Victim as mob in world)
			world	<< "[src] punches [Victim]."

			gTriggerPunches.events_Triggered(gEventHolder,src,Victim)
			gTriggerPunched.events_Triggered(gEventHolder,Victim,src)

//////////////////////////////////////////////////////////////////////////////////////////////////

//
//	Use an events_Holder to keep track of what things can be triggered to perform events,
//	what things can be triggered to add events, and what things can be triggered to remove events.
//

//////////////////////////////////////////////////////////////////////////////////////////////////
mob

	//============================================================================================
	New(Loc)
		..(Loc)
		//	When hugged, smile.
		//	The Caller and CallerSource are null because this event
		//	is being added manually instead of by a trigger.
		gEventHolder.events_AddEvent(src,gTriggerHugged,gEventSmile,":)",null,null)

		//	When punched, yell that the next offender is mean.
		//	I'm not using Argument for the event
		//	However, I'm writing in null so you know where it goes in the proc.
		gEventHolder.events_AddEventAddition(src,gTriggerPunched,gEventMean,gTriggerPunches,null)

		//	When hugged, stop calling the next offender mean.
		//	...Don't try this logic in real relationships. ;)
		//	Note the event will be removed even if the argument is not the same as what was added.
		gEventHolder.events_AddEventRemoval(src,gTriggerHugged,gEventMean,gTriggerPunches,"I'm different!")

	//--------------------------------------------------------------------------------------------
	Del()
		//	Remember to clean up after yourself.
		//	It probably won't break if you forget, but it will store a lot of useless data.
		gEventHolder.events_RemoveSource(src)
		..()

//////////////////////////////////////////////////////////////////////////////////////////////////