/*
===========================
Events library by ACWraith.
===========================

This library sacrifices some speed to store dynamic data on events and triggers.

The bulk of this library has been used in PathWrath.
However, limited support for event arguments has been added.
For example, if you have an event which changes an attribute
then you can send in the value to change it by.
However, a source object can only have one argument per event/trigger pair.

To Use:
	Create an events_Holder to keep track of what things can be triggered to perform events,
	what things can be triggered to add events, and what things can be triggered to remove events.

	Create events with events_Event.  (Mainly, you derive a type and define what events_Perform() does.)

	Create triggers with events_Trigger and call their events_Triggered() procs in your code where you
	want events	activated, added, or removed.

//////////////////////////////////////////////////////////////////////////////////////////////////
events_Event

	events_Perform(events_Holder/Holder,events_Trigger/Trigger,Source,Target,Argument)
		This is what is called for the event to do what you tell it to.
		Trigger is the events_Trigger which called the event.
		Holder is the container used to pair off the trigger and the event.
		Source is the owner of the event.  Target is what triggered the event.
		Argument is a recorded value for you to use to which is passed in automatically.
		This proc returns whether or not the event was performed.
		By default, it returns FALSE.

//////////////////////////////////////////////////////////////////////////////////////////////////
events_Trigger

	events_Triggered(events_Holder/Holder,Source,Target)
		This should be the only proc that your main code needs when referencing an events_Trigger.
		Calling this means Target activated this trigger for Source.
		By default this calls the following procs in the following order:
			events_TriggeredEvents()
			events_TriggeredRemovals()
			events_TriggeredAdditions()

	events_TriggeredAdditions(events_Holder/Holder,Source,Target)
		Holder is told to make the proper additions for when Target activates Source with this trigger.

	events_TriggeredEvents(events_Holder/Holder,Source,Target)
		Holder is told to perform the proper events for when Target activates Source with this trigger.

	events_TriggeredRemovals(events_Holder/Holder,Source,Target)
		Holder is told to make the proper removals for when Target activates Source with this trigger.

//////////////////////////////////////////////////////////////////////////////////////////////////
events_Holder

	events_RemoveSource(Source)
		Clear out all events, event additions, and event removals used by Source.

	==============================================================================================
	EVENTS
	==============================================================================================

	events_AddEvent(Source, events_Trigger/Trigger, events_Event/Event, Argument
			events_Trigger/Caller=null, CallerSource=null)
		Source should now have Event occur with Argument during Trigger.
		Returns FALSE if Source was already listed with the event/trigger pair (regardless of Argument).

		A user might want to make a derivative be more selective of what is added.
		For this reason, Caller (the calling trigger) is passed in along with
		CallerSource (what the trigger activated).  However, they do nothing by default.

	events_GetEvents(Source,events_Trigger/Trigger)
		Returns a list of events_Events (associated with the arguments Source calls them with)
		which occur when Trigger happens with Source.

	events_GetEventArgument(Source,events_Trigger/Trigger,events_Event/Event)
		Get the argument Source uses when calling Event during Trigger.

	events_HasEvent(Source,events_Trigger/Trigger,events_Event/Event)
		Returns whether or not Source has Event during Trigger.

	events_RemoveEvent(Source, events_Trigger/Trigger, events_Event/Event, Argument
			events_Trigger/Caller=null, CallerSource=null)
		Make Source no longer have Event occur during Trigger.
		By default, Argument is passed in, but ignored.
		Returns TRUE only if the event was indeed removed.
		(If it was not there in the first place then it returns FALSE.)

		A user might want to make a derivative be more selective of what is removed.
		For this reason, Caller (the calling trigger) is passed in along with
		CallerSource (what the trigger activated).  However, they do nothing by default.

	==============================================================================================
	EVENT ADDITIONS
	==============================================================================================

	events_AddEventAddition(Source, events_Trigger/Trigger,
			events_Event/AddedEvent, events_Trigger/AddTrigger, Argument)
		During Trigger, Source adds AddedEvent to happen during AddTrigger.
		Returns FALSE if the addition was already listed, regardless of Argument.

	events_GetAdditions(Source, events_Trigger/Trigger)
		Returns what Source adds during Trigger.
		The result is a list of events_Triggers associated with events_Events associated with arguments.

	events_GetAdditionArgument(Source, events_Trigger/Trigger,
			events_Event/AddedEvent, events_Trigger/AddTrigger)
		Get argument Source uses when adding Event for AddTrigger during Trigger.

	events_HasAddition(Source, events_Trigger/Trigger, events_Event/AddedEvent, events_Trigger/AddTrigger)
		Returns whether or not Source, during Trigger, will add Event to occur during AddTrigger.

	events_RemoveEventAddition(Source, events_Trigger/Trigger,
			events_Event/AddedEvent, events_Trigger/AddTrigger, Argument)

		During Trigger, Source no longer adds AddedEvent to happen during AddTrigger.
		Returns TRUE only if the removal occured.
		(If it was not there in the first place then it returns FALSE.)
		By default, Argument is passed in, but does nothing.

	==============================================================================================
	EVENT REMOVALS
	==============================================================================================

	events_AddEventRemoval(Source, events_Trigger/Trigger,
			events_Event/RemovedEvent, events_Trigger/RemoveTrigger, Argument)
		During Trigger, Source removes RemovedEvent during RemoveTrigger.
		Returns FALSE if it was already listed.
		By Default, Argument is passed in, but does nothing.

	events_GetRemovals(Source, events_Trigger/Trigger)
		Returns what Source removes during Trigger.
		The result is a list of events_Triggers associated with events_Events associated with arguments.

	events_GetRemovalArgument(Source, events_Trigger/Trigger,
			events_Event/RemovedEvent, events_Trigger/RemoveTrigger)
		Get argument Source uses when removing RemovedEvent for RemoveTrigger during Trigger.

	events_HasRemoval(Source, events_Trigger/Trigger,
			events_Event/RemovedEvent, events_Trigger/RemoveTrigger)
		Returns whether or not Source, during trigger, removes Event during RemoveTrigger.

	events_RemoveEventRemoval(Source, events_Trigger/Trigger,
			events_Event/RemovedEvent, events_Trigger/RemoveTrigger, Argument)
		During Trigger, Source no longer removes RemovedEvent during RemoveTrigger.
		Returns TRUE only if the removal occured.
		(If it was not there in the first place then it returns FALSE.)
		By default, Argument is passed in, but does nothing.

//////////////////////////////////////////////////////////////////////////////////////////////////
*/