//////////////////////////////////////////////////////////////////////////////////////////////////
events_Holder

	//============================================================================================
	var
		list
			events_removes		//	removes[Trigger][Trigger][Event][Source]	= Argument

	//============================================================================================
	proc
		//----------------------------------------------------------------------------------------
		//	During Trigger, Source removes RemovedEvent during RemoveTrigger.
		//	Returns FALSE if it was already listed.
		events_AddEventRemoval(Source, \
			events_Trigger/Trigger,events_Event/RemovedEvent,events_Trigger/RemoveTrigger,Argument)
			if ( isnull(Source) || isnull(Trigger) || isnull(RemovedEvent)  || isnull(RemoveTrigger) )
				return FALSE

			if ( isnull(src.events_removes) )
				src.events_removes	= list()
			if ( !(Trigger in src.events_removes) )
				src.events_removes[Trigger]	= list()
			if ( !(RemoveTrigger in src.events_removes[Trigger]) )
				src.events_removes[Trigger][RemoveTrigger]	= list()
			if ( !(RemovedEvent in src.events_removes[Trigger][RemoveTrigger]) )
				src.events_removes[Trigger][RemoveTrigger][RemovedEvent]	= list()
			if ( !(Source in src.events_removes[Trigger][RemoveTrigger][RemovedEvent]) )
				src.events_removes[Trigger][RemoveTrigger][RemovedEvent][Source]	= Argument
				return TRUE

			//return TRUE
			return FALSE

		//----------------------------------------------------------------------------------------
		//	Returns what Source removes during Trigger.
		//	The result is a list of events_Triggers associated with events_Events which are associated with arguments.
		events_GetRemovals(Source,events_Trigger/Trigger)
			if ( isnull(Source) || isnull(Trigger) )	return list()
			if ( !(Trigger in src.events_removes) )			return list()

			var
				results		= list()
				RemovedEvent
				RemoveTrigger
			for(RemoveTrigger in src.events_removes[Trigger])
				for(RemovedEvent in src.events_removes[Trigger][RemoveTrigger])
					if (Source in src.events_removes[Trigger][RemoveTrigger][RemovedEvent])
						if ( !(RemoveTrigger in results) )
							results[RemoveTrigger]	= list()
						results[RemoveTrigger][RemovedEvent]	= src.events_removes[Trigger][RemoveTrigger][RemovedEvent][Source]

			return results

		//----------------------------------------------------------------------------------------
		//	Get argument Source uses when removing RemovedEvent for RemoveTrigger during Trigger.
		events_GetRemovalArgument(Source, \
			events_Trigger/Trigger,events_Event/RemovedEvent,events_Trigger/RemoveTrigger)
			if ( !src.events_HasRemoval(Source,Trigger,RemovedEvent,RemoveTrigger) )	return null
			return src.events_removes[Trigger][RemoveTrigger][RemovedEvent][Source]

		//----------------------------------------------------------------------------------------
		//	Returns whether or not Source, during trigger, removes Event during RemoveTrigger.
		events_HasRemoval(Source, \
			events_Trigger/Trigger,events_Event/RemovedEvent,events_Trigger/RemoveTrigger)
			if ( isnull(Source) || isnull(Trigger) || isnull(RemovedEvent)  || isnull(RemoveTrigger) )
				return FALSE
			if ( isnull(src.events_removes) )										return FALSE
			if ( !(Trigger in src.events_removes) )								return FALSE
			if ( !(RemoveTrigger in src.events_removes[Trigger]) )					return FALSE
			if ( !(RemovedEvent in src.events_removes[Trigger][RemoveTrigger]) )	return FALSE

			return (Source in src.events_removes[Trigger][RemoveTrigger][RemovedEvent])

		//----------------------------------------------------------------------------------------
		//	During Trigger, Source no longer removes RemovedEvent during RemoveTrigger.
		//	Returns TRUE only if the removal occured.
		//	(If it was not there in the first place then it returns FALSE.)
		//	Argument is passed in, but does nothing by default.
		events_RemoveEventRemoval(Source, \
			events_Trigger/Trigger,events_Event/RemovedEvent,events_Trigger/RemoveTrigger,Argument)
			if ( isnull(Source) || isnull(Trigger) || isnull(RemovedEvent)  || isnull(RemoveTrigger) )
				return FALSE
			if ( isnull(src.events_removes) )
				return FALSE
			if ( !(Trigger in src.events_removes) )
				return FALSE
			if ( !(RemoveTrigger in src.events_removes[Trigger]) )
				return FALSE
			if ( !(RemovedEvent in src.events_removes[Trigger][RemoveTrigger]) )
				return FALSE
			if ( !(Source in src.events_removes[Trigger][RemoveTrigger][RemovedEvent]) )
				return FALSE

			src.events_removes[Trigger][RemoveTrigger][RemovedEvent]	-= Source

			if ( !length(src.events_removes[Trigger][RemoveTrigger][RemovedEvent]) )
				src.events_removes[Trigger][RemoveTrigger]			-= RemovedEvent
			if ( !length(src.events_removes[Trigger][RemoveTrigger]) )
				src.events_removes[Trigger]							-= RemoveTrigger
			if ( !length(src.events_removes[Trigger]) )
				src.events_removes										-= Trigger
			if ( !length(src.events_removes) )
				src.events_removes										-= null

			return TRUE

//////////////////////////////////////////////////////////////////////////////////////////////////